// Author      : Bartomiej Szwajkowski 5SST


/**
 * Generic Image Library (boost::gil) jest to otwarta, generyczna biblioteka  przeznaczona do
 * uywania w programach do obrbki graficznej. Generyczno biblioteki oznacza i kod biblioteki
 * jest niezaleny od typu obrazu, inaczej mwic algorytmy zastosowane w GIL s na tyle oglne i
 * pozwalaj na operowanie na szerokiej gamie formatw obrazw.
 * Jej autorami s Panowie Lubomir Bourdev oraz Hailin Jin, pracownicy firmy Adobe.
 * Generic Image Library sta si czci biblioteki Boost poczwszy od
 * wersji 1.35.0 (29 Marzec 2008). Obecna wersja GIL to 2.1.1.
 *
 * Niniejszy tekst ma na celu zaprezentowanie moliwoci jakie oferuje GIL w zakresie konwersji
 * pomidzy najpopularniejszymi formatami plikw graficznych - JPEG, PNG oraz TIFF. Autorzy wyraaj
 * nadziej i w przyszoci zakres obsugiwanych formatw zostanie rozszerzony.
 */
#include <iostream>
#include <boost/mpl/vector.hpp>
#include <boost/gil/extension/dynamic_image/any_image.hpp>
#include <boost/gil/extension/io/jpeg_dynamic_io.hpp>
#include <boost/gil/extension/io/png_dynamic_io.hpp>
#include <boost/gil/extension/io/tiff_dynamic_io.hpp>

using namespace boost::gil;

int main() {

/**
 * Konwersjie pomidzy formatami plikw sprowadzj si przy zastosowaniu biblioteki GIL do wczytania posiadanego pliku a
 * nastpnie do zapisania w danym formacie.
 * Jak wspomniano na wstpie GIL potrafi obsugiwa obrazy niezalenie od formatu ich zapisu. Gwnymi parametrami
 * s przestrze kolorw (RGB, CMYK, etc.) oraz gbia kolorw (8bit, 16bit, etc). Std te do wczytywania plikw z dysku
 * oferuje szereg przecionych metod. Dziel si one na trzy przypadki:
 *  - gdy znamy dokadny format zapisanego pliku
 *  - gdy nie znamy modelu zapisu barw oraz gbi kolorw GIL odczyta je za nas i utworzy automatycznie obiekt zgodny z orginaem
 *  - gdy nie znamy w/w cech i chcemy stworzy obiekt o zadanych przez nas parametrach
 *
 *  W celu zachowania uniwersalnoci tego opisu w podanym przykadzie zostanie zaprezentowany przypadek drugi
 */

//	Tworzymy vector z przykadowymi typami obrazw - 8 i 16 bitowe odcienie szaroci oraz 8 i 16 bitowe
//	obrazy typu RGB
	typedef boost::mpl::vector<gray8_image_t, rgb8_image_t, gray16_image_t,
			rgb16_image_t> dynamic_image_t;
//	klasa any_image pozwala przechowa zdjcie dowolnego typu zawartego w vectorze dynamic_image_t
	any_image<dynamic_image_t> dynamic_jpeg_image;

//	wczytanie do wczeniej zadeklarowanego obiekty Image obrazu typu JPEG
//	dla kadego z obsugiwanych typw instniej oddzielne metody wczytujce
	std::cout << "Wczytywanie pliku typu JPEG" << std::endl;
	jpeg_read_image("test.jpg", dynamic_jpeg_image);

//    typedef boost::mpl::vector<gray8_view_t, gray16_view_t, rgb8_view_t, rgb16_view_t> my_views_t;
//    any_image_view<my_views_t> dynamic_view;
//    dynamic_view = view(dynamic_jpeg_image);
//    std::cout <<  dynamic_view.dimensions() << std::endl;
	any_image<dynamic_image_t> dynamic_png_image;

//	wczytanie do wczeniej zadeklarowanego obiekty Image obrazu typu PNG
	std::cout << "Wczytywanie pliku PNG" << std::endl;
	png_read_image("test.png", dynamic_png_image);


	any_image<dynamic_image_t> dynamic_tiff_image;
	//	wczytanie do wczeniej zadeklarowanego obiekty Image obrazu typu TIFF
	std::cout << "Wczytywanie pliku TIFF" << std::endl;
	tiff_read_image("test.tiff", dynamic_tiff_image);

//	Analogicznie do procesu wczytywania dokonuje si zapisu do pliku  po przez wywoanie odpowiedniej
//	do dnego rozszerzenia metody, w celu zapisania naley utworzy view (odpowiednik zakresw
//	w biblitece STL) danego obrazu
	std::cout << "Zapis do pliku JPEG" << std::endl;
	jpeg_write_view("out-tiff-to-jpeg.jpeg", const_view(dynamic_tiff_image));
	std::cout << "Zapis do pliku PNG" << std::endl;
	png_write_view("out-jpeg-to-png.png", const_view(dynamic_jpeg_image));
	std::cout << "Zapis do pliku TIFF" << std::endl;
	tiff_write_view("out-png-to-tiff.tiff", const_view(dynamic_png_image));

	return 0;
}
